import unittest

import polyfempy as pf
from .utils import plot
import os


class BendingTest(unittest.TestCase):
    def test_run(self):
        dir_path = os.path.dirname(os.path.realpath(__file__))
        mesh_path = os.path.join(dir_path, "../3rdparty/data/plane_hole.obj")

        settings = pf.Settings()
        settings.discr_order = 1
        settings.normalize_mesh = True
        # settings.vismesh_rel_area = 0.1


        settings.set_material_params("E", 210000)
        settings.set_material_params("nu", 0.3)


        settings.tensor_formulation = pf.TensorFormulations.LinearElasticity

        problem = pf.GenericTensor()
        problem.add_dirichlet_value(1, [0, 0], [True, False])
        problem.add_dirichlet_value(4, [0, 0], [False, True])

        problem.add_neumann_value(3, [100, 0])

        settings.set_problem(problem)


        solver = pf.Solver()

        solver.settings(str(settings))
        solver.load_mesh(mesh_path)

        solver.solve()

        [pts, tets, disp] = solver.get_sampled_solution()
        vertices = pts + disp
        mises, _ = solver.get_sampled_mises_avg()

        plot(vertices, tets, mises)



if __name__ == '__main__':
    unittest.main()



import json


class Settings:
	"""Class that encodes the settings of the solver, it nodels the input json file"""

	def __init__(self):
		self.discr_order = 1
		self.pressure_discr_order = 1

		self.scalar_formulation = "Laplacian"
		self.tensor_formulation = "LinearElasticity"

		self.normalize_mesh = True

		self.n_refs = 0
		self.vismesh_rel_area = 0.00001
		self.nl_solver_rhs_steps = 1



		self.problem = "Franke"


		self.tend = 1
		self.time_steps = 10


		self.quadrature_order = 4

		self.params = {}
		self.export = {}
		self.advanced_options = {}

		self.problem_params = {}


	def set_problem(self, problem):
		"""Sets the problem, use any of the problems in Problems"""

		self.problem = problem.name()
		self.problem_params = problem.params()


	def set_material_params(self, name, value):
		"""set the material parameters, for instance set_material_params("E", 200) sets the Young's modulus E to 200. See https://polyfem.github.io/documentation/#formulations for full list"""

		self.params[name] = value


	def set_vtu_export_path(self, path, bounda_only=False):
		"""Sets the path to export a vtu file with the results, use bounda_only to export only one layer of the mesh in 3d"""

		self.export["vis_mesh"] = path
		self.export["vis_boundary_only"] = bounda_only


	def set_wireframe_export_path(self, path):
		"""Sets the path to export a wireframe of the mesh"""

		self.export["wire_mesh"] = path


	def set_isolines_export_path(self, path):
		"""Sets the path to export the isolines of the solution"""

		self.export["iso_mesh"] = path


	def set_solution_export_path(self, path):
		"""Sets the path to save the solution"""

		self.export["solution"] = path


	def set_advanced_option(self, key, value):
		"""Used to set any advanced option not present in this class, for instance set_advanced_option("use_spline",True), see https://polyfem.github.io/documentation/ for full list"""

		self.advanced_options[key] = value

	def __str__(self):
		"""stringygied json description of this class, used to run the solver"""

		tmp = dict(
			(key, value)
			for (key, value) in self.__dict__.items())
		tmp.pop('advanced_options', None)
		tmp.update(self.advanced_options)

		return json.dumps(tmp, sort_keys=True, indent=4)

	def serialize(self):
		"""stringygied json description of this class, used to run the solver"""

		return str(self)



import unittest

import numpy as np
import polyfempy as pf
from utils import plot


class Gravity(unittest.TestCase):
    def test_run(self):
        n_pts = 3

        extend = np.linspace(0, 1, n_pts)
        x, y = np.meshgrid(extend, extend, sparse=False, indexing='xy')
        pts = np.column_stack((x.ravel(), y.ravel()))


        # Create connectivity
        faces = np.ndarray([(n_pts-1)**2, 4],dtype=np.int32)

        index = 0
        for i in range(n_pts-1):
            for j in range(n_pts-1):
                faces[index, :] = np.array([
                    j + i * n_pts,
                    j+1 + i * n_pts,
                    j+1 + (i+1) * n_pts,
                    j + (i+1) * n_pts
                ])
                index = index + 1

        # create settings
        settings = pf.Settings()
        settings.discr_order = 1

        settings.set_material_params("E", 2100)
        settings.set_material_params("nu", 0.3)

        # We use a linear material model
        settings.tensor_formulation = pf.TensorFormulations.LinearElasticity

        problem = pf.Gravity()
        settings.set_problem(problem)

        solver = pf.Solver()
        solver.settings(str(settings))

        # This time we are using pts and faces instead of loading from the disk
        solver.set_mesh(pts, faces)

        solver.solve()

        #Animation frame, last one
        frame = -1

        # Get the solution
        pts = solver.get_sampled_points_frames()
        tets = solver.get_sampled_connectivity_frames()
        disp = solver.get_sampled_solution_frames()


        # diplace the mesh
        vertices = pts[frame] + disp[frame]

        # and get the stresses
        mises = solver.get_sampled_mises_avg_frames()

        # finally plot
        plot(vertices, tets[frame], mises[frame])


if __name__ == '__main__':
    unittest.main()



from .polyfempy import Solver
from .polyfempy import ScalarFormulations
from .polyfempy import TensorFormulations

from .Settings import Settings

from .Problems import Franke
from .Problems import GenericScalar
from .Problems import Gravity
from .Problems import Torsion
from .Problems import GenericTensor
from .Problems import Flow
from .Problems import DrivenCavity



import plotly.offline as plotly
import plotly.graph_objs as go

import numpy as np


def plot(vertices, connectivity, function):
    x = vertices[:,0]
    y = vertices[:,1]

    if vertices.shape[1] == 3:
        z = vertices[:,2]
    else:
        z = np.zeros(x.shape, dtype=x.dtype)

    if connectivity.shape[1] == 3:
        f = connectivity
    else:
        f = np.ndarray([len(connectivity)*4, 3], dtype=np.int64)
        for i in range(len(connectivity)):
            f[i*4+0] = np.array([connectivity[i][1], connectivity[i][0], connectivity[i][2]])
            f[i*4+1] = np.array([connectivity[i][0], connectivity[i][1], connectivity[i][3]])
            f[i*4+2] = np.array([connectivity[i][1], connectivity[i][2], connectivity[i][3]])
            f[i*4+3] = np.array([connectivity[i][2], connectivity[i][0], connectivity[i][3]])

    mesh = go.Mesh3d(x=x, y=y, z=z,
                     i=f[:,0], j=f[:,1], k=f[:,2],
                     intensity=function, flatshading=function is not None,
                     colorscale='Viridis',
                     contour=dict(show=True, color='#fff'),
                     hoverinfo="all")
    layout = go.Layout(scene=go.layout.Scene(aspectmode='data'))
    fig = go.Figure(data=[mesh], layout=layout)

    plotly.plot(fig)



import unittest

import polyfempy as pf
import numpy as np

from .utils import plot

import os


class InflationTest(unittest.TestCase):
	def test_run(self):
		solver = pf.Solver()

		#some setup
		dir_path = os.path.dirname(os.path.realpath(__file__))
		mesh_path = os.path.join(dir_path, "../3rdparty/data/circle2.msh")
		print(mesh_path)

		settings = pf.Settings()
		settings.discr_order = 2
		settings.normalize_mesh = True
		settings.vismesh_rel_area = 0.00001
		settings.scalar_formulation = pf.ScalarFormulations.Laplacian

		problem = pf.GenericScalar()
		problem.add_dirichlet_value("all", 10)
		problem.rhs = 0
		settings.set_problem(problem)



		solver.settings(str(settings))
		solver.load_mesh(mesh_path)

		solver.solve()
		sol = solver.get_solution()
		##########################################################################

		# now we got the solution of the first laplacian, we use it as rhs for the second one
		# setup zero bc and use sol as rhs
		problem = pf.GenericScalar()
		problem.add_dirichlet_value("all", 0)
		problem.rhs = 0
		settings.set_problem(problem)

		#reload the parameters and mesh
		solver.settings(settings.serialize())
		solver.load_mesh(mesh_path)

		#set the rhs as prev sol
		solver.set_rhs(sol)

		solver.solve()

		#get the solution on a densly sampled mesh
		[vertices, tris, sol] = solver.get_sampled_solution()



		#the dense mesh is disconnected, so we need to connecit it back
		_, res = np.unique(vertices, axis=0, return_inverse=True)
		vertices, resi = np.unique(vertices, axis=0, return_index=True)

		faces = np.ndarray([len(tris), 3], dtype=np.int64)
		vv = np.ndarray([len(vertices), 3], dtype=np.float64)

		for i in range(len(tris)):
			faces[i] = np.array([res[tris[i][0]], res[tris[i][1]], res[tris[i][2]]])

		for i in range(len(vv)):
			vv[i] = np.array([vertices[i][0], vertices[i][1], sol[resi[i]][0]])

		plot(vv, faces, None)

		# #save obj
		# with open(output, "w") as file:
		# 	# use sol as z
		# 	for i in range(len(vertices)):
		# 		file.write("v {} {} {}\n".format())

		# 	for i in range(len(tris)):
		# 		i0 = res[tris[i][0]]
		# 		i1 = res[tris[i][1]]
		# 		i2 = res[tris[i][2]]
		# 		file.write("f {} {} {}\n".format(i0+1, i1+1, i2+1))


if __name__ == '__main__':
	unittest.main()



import unittest

import polyfempy as pf
from .utils import plot

import os


class TorsionTest(unittest.TestCase):
    def test_run(self):
        dir_path = os.path.dirname(os.path.realpath(__file__))
        mesh_path = os.path.join(dir_path, "../3rdparty/data/square_beam_h.HYBRID")

        settings = pf.Settings()
        settings.discr_order = 1
        settings.normalize_mesh = False



        settings.set_material_params("E", 200)
        settings.set_material_params("nu", 0.35)

        settings.nl_solver_rhs_steps = 5
        settings.tensor_formulation = pf.TensorFormulations.NeoHookean

        problem = pf.Torsion()
        problem.fixed_boundary = 5
        problem.turning_boundary = 6
        problem.axis_coordiante = 2
        problem.n_turns = 0.5


        settings.vismesh_rel_area = 0.001

        settings.set_problem(problem)


        solver = pf.Solver()


        solver.settings(str(settings))
        solver.load_mesh(mesh_path)

        solver.solve()

        [pts, tets, disp] = solver.get_sampled_solution()
        vertices = pts + disp
        mises, _ = solver.get_sampled_mises_avg()

        plot(vertices, tets, mises)


if __name__ == '__main__':
    unittest.main()





#################################################
############Scalar###############################
class Franke:
	"""Franke problem with exact solution https://polyfem.github.io/documentation/#franke"""

	def name(self):
		return "Franke"

	def params(self):
		return {}


class GenericScalar:
	"""Generic scalar problem https://polyfem.github.io/documentation/#genericscalar"""

	def __init__(self):
		self.rhs = 0
		self.dirichlet_boundary = []
		self.neumann_boundary = []

	def add_dirichlet_value(self, id, value):
		"""add the Dirichlet value value for the sideset id"""

		tmp = {}
		tmp["id"] = id
		tmp["value"] = value
		self.dirichlet_boundary.append(tmp)

	def add_neumann_value(self, id, value):
		"""add the Neumann value value for the sideset id"""

		tmp = {}
		tmp["id"] = id
		tmp["value"] = value
		self.neumann_boundary.append(tmp)

	def name(self):
		return "GenericScalar"

	def params(self):
		return self.__dict__


#################################################
############Elasticity###########################
class Gravity:
	"""time dependent gravity problem https://polyfem.github.io/documentation/#gravity"""

	def name(self):
		return "Gravity"

	def params(self):
		return {}


class Torsion:
	"""3D torsion problem, specify which sideset to fix (fixed_boundary) and which one turns turning_boundary https://polyfem.github.io/documentation/#torsionelastic"""

	def __init__(self):
		self.axis_coordiante = 2
		self.n_turns = 0.5
		self.fixed_boundary = 5
		self.turning_boundary = 6


	def name(self):
		return "TorsionElastic"


	def params(self):
		return self.__dict__


class GenericTensor:
	"""Generic tensor problem https://polyfem.github.io/documentation/#generictensor"""

	def __init__(self):
		self.rhs = [0, 0, 0]
		self.dirichlet_boundary = []
		self.neumann_boundary = []

	def add_dirichlet_value(self, id, value, is_dirichlet_dim=None):
		"""add the Dirichlet value value for the sideset id. Note the value must be a vector in 2D or 3D depending on the problem. is_dirichlet_dim is a vector of boolean specifying which dimentions are fixed."""
		assert(len(value) == 3 or len(value) == 2)
		tmp = {}
		tmp["id"] = id
		tmp["value"] = value
		if is_dirichlet_dim is not None:
			assert(len(is_dirichlet_dim) == 3 or len(is_dirichlet_dim) == 2)
			tmp["dimension"] = is_dirichlet_dim

		self.dirichlet_boundary.append(tmp)

	def add_neumann_value(self, id, value):
		"""add the Neumann value value for the sideset id. Note the value must be a vector in 2D or 3D depending on the problem"""

		tmp = {}
		tmp["id"] = id
		tmp["value"] = value
		self.neumann_boundary.append(tmp)

	def name(self):
		return "GenericTensor"

	def params(self):
		return self.__dict__


#################################################
############Stokes###############################
class Flow:
	"""Inflow/outflow problem for fluids. You can specify the sideset for the moving fluxes and the list of obstacle sidesets. https://polyfem.github.io/documentation/#flow"""

	def __init__(self):
		self.inflow = 1
		self.outflow = 3
		self.inflow_amout = 0.25
		self.outflow_amout = 0.25
		self.obstacle = [7]

	def name(self):
		return "Flow"

	def params(self):
		return self.__dict__


class DrivenCavity:
	"""Classical driven cavity problem in fluid simulation"""

	def name(self):
		return "DrivenCavity"

	def params(self):
		return {}
